"WARNING: this file was automatically generated by commented code within mathpad.py, and then modified slightly"
import sympy.physics.units.definitions.unit_definitions as u
from mathpad.val import OutputVal

atomic_mass_constant = OutputVal(u.atomic_mass_constant, 1)
avogadro_constant = OutputVal(u.avogadro_constant, 1)
boltzmann_constant = OutputVal(u.boltzmann_constant, 1)
coulomb_constant = OutputVal(u.coulomb_constant, 1)
coulombs_constant = OutputVal(u.coulombs_constant, 1)
electric_constant = OutputVal(u.electric_constant, 1)
electric_force_constant = OutputVal(u.electric_force_constant, 1)
faraday_constant = OutputVal(u.faraday_constant, 1)
gravitational_constant = OutputVal(u.gravitational_constant, 1)
josephson_constant = OutputVal(u.josephson_constant, 1)
magnetic_constant = OutputVal(u.magnetic_constant, 1)
molar_gas_constant = OutputVal(u.molar_gas_constant, 1)
stefan_boltzmann_constant = OutputVal(u.stefan_boltzmann_constant, 1)
von_klitzing_constant = OutputVal(u.von_klitzing_constant, 1)