"WARNING: this file was automatically generated by commented code within mathpad.py, and then modified slightly"
import sympy.physics.units.definitions.unit_definitions as u
from mathpad.dimensions import *

A = Current(u.A)
Bq = Frequency(u.Bq)
C = Charge(u.C)
D = Dioptre(u.D)
F = Capacitance(u.F)
G = GravityConstant(u.G)
Gy = Gray(u.Gy)
H = Inductance(u.H)
Hz = Frequency(u.Hz)
J = Energy(u.J)
K = Temperature(u.K)
N = Force(u.N)
Pa = EnergyDensity(u.Pa)
R = MolarGasConstant(u.R)
S = Conductance(u.S)
T = MagneticDensity(u.T)
V = Voltage(u.V)
W = Power(u.W)
Wb = MagneticFlux(u.Wb)
Z0 = Impedance(u.Z0)
acceleration_due_to_gravity = Acceleration(u.acceleration_due_to_gravity)
ampere = Current(u.ampere)
amperes = Current(u.amperes)
amu = Mass(u.amu)
amus = Mass(u.amus)
rad = Angle(u.rad)
angular_mil = AngularMil(u.angular_mil)
angular_mils = AngularMil(u.angular_mils)
anomalistic_year = Time(u.anomalistic_year)
anomalistic_years = Time(u.anomalistic_years)
astronomical_unit = Length(u.astronomical_unit)
astronomical_units = Length(u.astronomical_units)
atm = EnergyDensity(u.atm)
atmosphere = EnergyDensity(u.atmosphere)
atmospheres = EnergyDensity(u.atmospheres)
atomic_mass_unit = Mass(u.atomic_mass_unit)
au = Length(u.au)
avogadro_number = Dimensionless(u.avogadro_number)
bar = EnergyDensity(u.bar)
bars = EnergyDensity(u.bars)
becquerel = Frequency(u.becquerel)
bit = Information(u.bit)
bits = Information(u.bits)
byte = Information(u.byte)
c = Velocity(u.c)
candela = LuminousIntensity(u.candela)
candelas = LuminousIntensity(u.candelas)
cd = LuminousIntensity(u.cd)
centiliter = Volume(u.centiliter)
centiliters = Volume(u.centiliters)
centimeter = Length(u.centimeter)
centimeters = Length(u.centimeters)
cl = Volume(u.cl)
cm = Length(u.cm)
common_year = Time(u.common_year)
common_years = Time(u.common_years)
coulomb = Charge(u.coulomb)
coulombs = Charge(u.coulombs)
curie = Frequency(u.curie)
day = Time(u.day)
days = Time(u.days)
deciliter = Volume(u.deciliter)
deciliters = Volume(u.deciliters)
decimeter = Length(u.decimeter)
decimeters = Length(u.decimeters)
deg = Dimensionless(u.deg)
degree = Dimensionless(u.degree)
degrees = Dimensionless(u.degrees)
dioptre = Dioptre(u.dioptre)
dl = Volume(u.dl)
dm = Length(u.dm)
draconic_year = Time(u.draconic_year)
draconic_years = Time(u.draconic_years)
e0 = Permittivity(u.e0)
eV = Energy(u.eV)
electronvolt = Energy(u.electronvolt)
electronvolts = Energy(u.electronvolts)
elementary_charge = Charge(u.elementary_charge)
exbibyte = Information(u.exbibyte)
exbibytes = Information(u.exbibytes)
farad = Capacitance(u.farad)
farads = Capacitance(u.farads)
feet = Length(u.feet)
foot = Length(u.foot)
ft = Length(u.ft)
full_moon_cycle = Time(u.full_moon_cycle)
full_moon_cycles = Time(u.full_moon_cycles)
g = Mass(u.g)
gaussian_year = Time(u.gaussian_year)
gaussian_years = Time(u.gaussian_years)
gee = Acceleration(u.gee)
gees = Acceleration(u.gees)
gibibyte = Information(u.gibibyte)
gibibytes = Information(u.gibibytes)
gram = Mass(u.gram)
grams = Mass(u.grams)
gray = Gray(u.gray)
h = Time(u.h)
hbar = Action(u.hbar)
henry = Inductance(u.henry)
henrys = Inductance(u.henrys)
hertz = Frequency(u.hertz)
hour = Time(u.hour)
hours = Time(u.hours)
hz = Frequency(u.hz)
inch = Length(u.inch)
inches = Length(u.inches)
joule = Energy(u.joule)
joules = Energy(u.joules)
julian_year = Time(u.julian_year)
julian_years = Time(u.julian_years)
kPa = EnergyDensity(u.kPa)
kat = Katal(u.kat)
katal = Katal(u.katal)
kelvin = Temperature(u.kelvin)
kelvins = Temperature(u.kelvins)
kg = Mass(u.kg)
kibibyte = Information(u.kibibyte)
kibibytes = Information(u.kibibytes)
kilogram = Mass(u.kilogram)
kilograms = Mass(u.kilograms)
kilometer = Length(u.kilometer)
kilometers = Length(u.kilometers)
kilopascal = EnergyDensity(u.kilopascal)
km = Length(u.km)
l = Volume(u.l)
lightyear = Length(u.lightyear)
lightyears = Length(u.lightyears)
liter = Volume(u.liter)
liters = Volume(u.liters)
lux = Illuminance(u.lux)
lx = Illuminance(u.lx)
ly = Length(u.ly)
m = Length(u.m)
mebibyte = Information(u.mebibyte)
mebibytes = Information(u.mebibytes)
meter = Length(u.meter)
meters = Length(u.meters)
mg = Mass(u.mg)
mho = Conductance(u.mho)
mhos = Conductance(u.mhos)
mi = Length(u.mi)
microgram = Mass(u.microgram)
micrograms = Mass(u.micrograms)
micrometer = Length(u.micrometer)
micrometers = Length(u.micrometers)
micron = Length(u.micron)
microns = Length(u.microns)
microsecond = Time(u.microsecond)
microseconds = Time(u.microseconds)
mil = AngularMil(u.mil)
mile = Length(u.mile)
miles = Length(u.miles)
milli_mass_unit = Mass(u.milli_mass_unit)
milligram = Mass(u.milligram)
milligrams = Mass(u.milligrams)
milliliter = Volume(u.milliliter)
milliliters = Volume(u.milliliters)
millimeter = Length(u.millimeter)
millimeters = Length(u.millimeters)
millisecond = Time(u.millisecond)
milliseconds = Time(u.milliseconds)
minute = Time(u.minute)
minutes = Time(u.minutes)
ml = Volume(u.ml)
mm = Length(u.mm)
mmHg = EnergyDensity(u.mmHg)
mmu = Mass(u.mmu)
mmus = Mass(u.mmus)
mol = AmountOfSubstance(u.mol)
mole = AmountOfSubstance(u.mole)
moles = AmountOfSubstance(u.moles)
ms = Time(u.ms)
nanometer = Length(u.nanometer)
nanometers = Length(u.nanometers)
nanosecond = Time(u.nanosecond)
nanoseconds = Time(u.nanoseconds)
nautical_mile = Length(u.nautical_mile)
nautical_miles = Length(u.nautical_miles)
newton = Force(u.newton)
newtons = Force(u.newtons)
nm = Length(u.nm)
nmi = Length(u.nmi)
ns = Time(u.ns)
ohm = Impedance(u.ohm)
ohms = Impedance(u.ohms)
optical_power = Dioptre(u.optical_power)
pa = EnergyDensity(u.pa)
pascal = EnergyDensity(u.pascal)
pascals = EnergyDensity(u.pascals)
pebibyte = Information(u.pebibyte)
pebibytes = Information(u.pebibytes)
percent = Dimensionless(u.percent)
percents = Dimensionless(u.percents)
permille = Dimensionless(u.permille)
picometer = Length(u.picometer)
picometers = Length(u.picometers)
picosecond = Time(u.picosecond)
picoseconds = Time(u.picoseconds)
planck = Action(u.planck)
planck_acceleration = Acceleration(u.planck_acceleration)
planck_angular_frequency = Frequency(u.planck_angular_frequency)
planck_area = Area(u.planck_area)
planck_charge = Charge(u.planck_charge)
planck_current = Current(u.planck_current)
planck_density = Density(u.planck_density)
planck_energy = Energy(u.planck_energy)
planck_energy_density = EnergyDensity(u.planck_energy_density)
planck_force = Force(u.planck_force)
planck_impedance = Impedance(u.planck_impedance)
planck_intensity = PlanckIntensity(u.planck_intensity)
planck_length = Length(u.planck_length)
planck_mass = Mass(u.planck_mass)
planck_momentum = Momentum(u.planck_momentum)
planck_power = Power(u.planck_power)
planck_pressure = EnergyDensity(u.planck_pressure)
planck_temperature = Temperature(u.planck_temperature)
planck_time = Time(u.planck_time)
planck_voltage = Voltage(u.planck_voltage)
planck_volume = Volume(u.planck_volume)
pm = Length(u.pm)
pound = Mass(u.pound)
pounds = Mass(u.pounds)
ps = Time(u.ps)
psi = EnergyDensity(u.psi)
quart = Volume(u.quart)
quarts = Volume(u.quarts)
rad = Angle(u.rad)
radian = Angle(u.radian)
radians = Angle(u.radians)
rutherford = Frequency(u.rutherford)
s = Time(u.s)
second = Time(u.second)
seconds = Time(u.seconds)
sidereal_year = Time(u.sidereal_year)
sidereal_years = Time(u.sidereal_years)
siemens = Conductance(u.siemens)
speed_of_light = Velocity(u.speed_of_light)
sr = SteRadian(u.sr)
steradian = SteRadian(u.steradian)
steradians = SteRadian(u.steradians)
tebibyte = Information(u.tebibyte)
tebibytes = Information(u.tebibytes)
tesla = MagneticDensity(u.tesla)
teslas = MagneticDensity(u.teslas)
torr = EnergyDensity(u.torr)
tropical_year = Time(u.tropical_year)
tropical_years = Time(u.tropical_years)
ug = Mass(u.ug)
um = Length(u.um)
us = Time(u.us)
v = Voltage(u.v)
vacuum_impedance = Impedance(u.vacuum_impedance)
vacuum_permittivity = Permittivity(u.vacuum_permittivity)
volt = Voltage(u.volt)
volts = Voltage(u.volts)
watt = Power(u.watt)
watts = Power(u.watts)
wb = MagneticFlux(u.wb)
weber = MagneticFlux(u.weber)
webers = MagneticFlux(u.webers)
yard = Length(u.yard)
yards = Length(u.yards)
yd = Length(u.yd)
year = Time(u.year)
years = Time(u.years)